import "Turbine.UI";

LPRollListItem = class(Turbine.UI.Control);

LPRollListItem.DefaultColor = Turbine.UI.Color(255/255, 255/255, 255/255);
LPRollListItem.DisabledColor = Turbine.UI.Color(75/255, 75/255, 75/255);
LPRollListItem.ReadyColor = Turbine.UI.Color(0, 127/255, 0);
LPRollListItem.OneColor = Turbine.UI.Color(127/255, 0, 0);
LPRollListItem.OutlineColor = Turbine.UI.Color(255/255, 255/255, 255/255);

LPRollListItem.DefaultBackColor = Turbine.UI.Color(.9, 0, 0, 0);
LPRollListItem.HighlightColor = Turbine.UI.Color(35/255, 36/255, 40/255);
LPRollListItem.SelectionColor = Turbine.UI.Color(153/255, 153/255, 153/255);

function LPRollListItem:Constructor(name)
	Turbine.UI.Control.Constructor(self);
	
	-- Name.
	self.lblName = Turbine.UI.Label();
	self.lblName:SetParent(self);
	self.lblName:SetPosition(2, 0);
	self.lblName:SetSize(125, 16);
	self.lblName:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.lblName:SetForeColor(LPRollListItem.DefaultColor);
	self.lblName:SetFont(Turbine.UI.Lotro.Font.TrajanPro15);
	self.lblName:SetText(name);
	
	-- Roll.
	self.txtRoll = Turbine.UI.TextBox();
	self.txtRoll:SetParent(self);
	self.txtRoll:SetPosition(127, 0);
	self.txtRoll:SetSize(28, 16);
	self.txtRoll:SetMultiline(false);
	self.txtRoll:SetSelectable(false);
	self.txtRoll:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
	self.txtRoll:SetForeColor(LPRollListItem.DefaultColor);
	self.txtRoll:SetFont(Turbine.UI.Lotro.Font.TrajanPro15);
	self.txtRoll:SetWantsKeyEvents(true);
	self.txtRoll:SetText("");
	
	-- Roll status.
	self.hasRolled = false;
	
	-- Roll edit status.
	self.editingRoll = false;
	
	-- Base event handlers.
	self.FocusGained = function(sender, args)
		self.lblName:SetBackColor(LPRollListItem.SelectionColor);
		self.txtRoll:SetBackColor(LPRollListItem.SelectionColor);
	end

	self.FocusLost = function(sender, args)
		self.lblName:SetBackColor(LPRollListItem.DefaultBackColor);
		self.txtRoll:SetBackColor(LPRollListItem.DefaultBackColor);
	end

	-- Name column event handlers.
	self.lblName.MouseEnter = function(sender, args)
		if not self:HasFocus() then
			self.lblName:SetBackColor(LPRollListItem.HighlightColor);
			self.txtRoll:SetBackColor(LPRollListItem.HighlightColor);
		end
	end

	self.lblName.MouseLeave = function(sender, args)
		if not self:HasFocus() then
			self.lblName:SetBackColor(LPRollListItem.DefaultBackColor);
			self.txtRoll:SetBackColor(LPRollListItem.DefaultBackColor);
		end
	end

	self.lblName.MouseDown = function(sender, args)
		if (args.Button == Turbine.UI.MouseButton.Right) then
			self:OpenMenu();
		end
		self:Focus();
	end
	
	-- Roll column event handlers.
	self.txtRoll.MouseEnter = function(sender, args)
		if not self:HasFocus() then
			self.lblName:SetBackColor(LPRollListItem.HighlightColor);
			self.txtRoll:SetBackColor(LPRollListItem.HighlightColor);
		end
	end

	self.txtRoll.MouseLeave = function(sender, args)
		if not self:HasFocus() then
			self.lblName:SetBackColor(LPRollListItem.DefaultBackColor);
			self.txtRoll:SetBackColor(LPRollListItem.DefaultBackColor);
		end
	end

	self.txtRoll.MouseDown = function(sender, args)
		if (args.Button == Turbine.UI.MouseButton.Right) then
			self:OpenMenu();
		end
		self:Focus();
	end
	
	self.txtRoll.KeyDown = function(sender, args)
		if self.editingRoll then
			-- Enter/return key.
			if (args.Action == 162) then
				self:RollBounds();
				
				-- Lose focus on the text box to prevent further editing.
				self.lblName:Focus();
			end
		end
	end
	
	self.txtRoll.FocusLost = function(sender, args)
		self:RollBounds();
	end
end

function LPRollListItem:OpenMenu()
	cmOptions = Turbine.UI.ContextMenu();
	cmItems = cmOptions:GetItems();
			
	cmItems:Add(Turbine.UI.MenuItem("Reset Roll Status"));
	cmItems:Add(Turbine.UI.MenuItem("Manually Set Roll"));

	cmItems:Get(1).Click = function(sender, args)
		self:ResetRoll();
	end

	cmItems:Get(2).Click = function(sender, args)
		self:ManualRoll();
	end

	cmOptions:ShowMenu();
end

function LPRollListItem:ResetRoll()
	self.hasRolled = false;
	self.txtRoll:SetText("");
	self:SetDefault();
	self:GetParent().MonitorRolls = true;
end

function LPRollListItem:ManualRoll()
	self.txtRoll:SetSelectable(true);
	self.txtRoll:SelectAll();
	self.txtRoll:Focus();
	self.editingRoll = true;
end

function LPRollListItem:RollBounds()
	-- Finished manually modifying the roll.  Check and validate value.
	if self.editingRoll then
		local val = tonumber(self.txtRoll:GetText());
		if ((val == nil) or (val < 1)) then
			val = "";
			self:SetDefault();
			self.hasRolled = false;
		else
			if (val > 100) then
				val = 100;
			end
			self.hasRolled = true;
		end
		self.txtRoll:SetText(val);
		self.txtRoll:SetSelectable(false);
		self.editingRoll = false;
	end
end

function LPRollListItem:GetName()
	return self.lblName:GetText();
end

function LPRollListItem:GetRoll()
	if (tonumber(self.txtRoll:GetText()) ~= nil) then
		return tonumber(self.txtRoll:GetText());
	else
		return 0
	end
end

function LPRollListItem:SetRoll(roll)
	if (tonumber(roll) == nil) or (tonumber(roll) < 1) then
		self.txtRoll:SetText("");
	elseif (tonumber(roll) > 100) then
		self.txtRoll:SetText("100");
		self.hasRolled = true;
	else
		self.txtRoll:SetText(roll);
		self.hasRolled = true;
	end
end

function LPRollListItem:HasRolled()
	return self.hasRolled;
end

function LPRollListItem:SetDefault()
	self.lblName:SetForeColor(LPRollListItem.DefaultColor);
end

function LPRollListItem:SetDisabled()
	self.lblName:SetForeColor(LPRollListItem.DisabledColor);
end

function LPRollListItem:SetReady()
	self.lblName:SetForeColor(LPRollListItem.ReadyColor);
end

function LPRollListItem:SetOne()
	self.txtRoll:SetFontStyle(Turbine.UI.FontStyle.Outline);
	self.txtRoll:SetOutlineColor(LPRollListItem.OneColor);
	--self.lblName:SetForeColor(LPRollListItem.OneColor);
end

function LPRollListItem:SetWinner()
	self.txtRoll:SetFontStyle(Turbine.UI.FontStyle.Outline);
	self.txtRoll:SetOutlineColor(LPRollListItem.OutlineColor);
end