import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Gameplay";
import "LyciusPlugins.Lib";
import "LyciusPlugins.RaidRolls";

RaidRolls = class(Turbine.UI.Lotro.Window);

RaidRolls.PanelColor = Turbine.UI.Color(.9, 0, 0, 0);

function RaidRolls:Constructor()
	Turbine.UI.Lotro.Window.Constructor(self);
	
	-- Load settings.
	self:LoadSettings();

	-- Window properties.
	self:SetSize(180, 290);
	self:SetBackColor(Turbine.UI.Color());
	self:SetPosition(self.settings.posX, self.settings.posY);
	self:SetText("Raid Rolls");
	self:SetWantsKeyEvents(true);

	-- Save position settings if necessary.
	self.PositionChanged = function(sender, args)
		local posX, posY = self:GetPosition();
		self.settings.posX = posX;
		self.settings.posY = posY;
		self:SaveSettings();
	end

	-- Monitor key presses.
	self.KeyDown = function(sender, args)
		-- Hide window when pressing escape key.
		if (args.Action == Turbine.UI.Lotro.Action.Escape) then
			sender:SetVisible(false);
		end
	end

	-- Flag to monitor for rolls.
	self.MonitorChat = false;
	
	lbRolls = Turbine.UI.ListBox();
	lbRolls:SetParent(self);
	lbRolls:SetBackColor(RaidRolls.PanelColor);
	lbRolls:SetPosition(10, 37);
	lbRolls:SetSize(160, 210);
      
	self:InitializeGroup();
	
	-- Give the tree view a scroll bar.
    sbRolls = Turbine.UI.Lotro.ScrollBar();
    sbRolls:SetOrientation(Turbine.UI.Orientation.Vertical);
    sbRolls:SetParent(self);
    sbRolls:SetPosition(168, 38);
    sbRolls:SetSize(10, 209);
    
    lbRolls:SetVerticalScrollBar(sbRolls);

	-- Open Rolls button.
	btnOpen = LyciusPlugins.Lib.LPButton("Open Rolls");
	btnOpen:SetParent(self);
	btnOpen:SetPosition(10, 257);
	btnOpen:SetSize(75, 18);

	btnOpen.Click = function(sender, args)
		self:InitializeGroup();
		self.MonitorRolls = true;
	end

	-- Close Rolls button.
	btnClose = LyciusPlugins.Lib.LPButton("Close Rolls");
	btnClose:SetParent(self);
	btnClose:SetPosition(95, 257);
	btnClose:SetSize(75, 18);

	btnClose.Click = function(sender, args)
		self:FinalizeGroup();
		self.MonitorRolls = false;
	end

	-- Chat monitor.
	ChatMonitor = Turbine.Chat;
	ChatMonitor.Received = function(f, args)
		if self.MonitorRolls then
			-- Valid if in Standard chat and sent by local player.
			if (args.ChatType == Turbine.ChatType.Standard) and (args.Sender == self.Player:GetName()) then
				local msg = args.Message;
				local name, roll = string.match(msg, "(.+) [%S]+ %(100%): (%d+)");
				
				-- Proceed if message matches the pattern.
				if (name ~= nil) then
					for i = 1, lbRolls:GetItemCount() do
						local listItem = lbRolls:GetItem(i);
						
						-- If name matches and they haven't yet been recorded, set their roll.
						if ((listItem:GetName() == name) and (listItem:HasRolled() == false)) then
							listItem:SetRoll(roll);
						end
					end
				end
			end
		end
	end
end

function RaidRolls:InitializeGroup()
	lbRolls:ClearItems();
	
	-- Keep track of local player for chat monitoring.
	self.Player = Turbine.Gameplay.LocalPlayer.GetInstance();
	
	local party = self.Player:GetParty();
	
	if (party ~= nil) then
		-- Rolling with a group.
		for i = 1, party:GetMemberCount() do
			local member = party:GetMember(i);
			
			-- Make sure member is valid.  Sometimes invalid names are returned as "<string table error>".
			if (string.find(member:GetName(), "<") == nil) then
				local listItem = LyciusPlugins.RaidRolls.LPRollListItem(member:GetName());
				listItem:SetSize(lbRolls:GetWidth(), 16);
				lbRolls:AddItem(listItem);
				--listItem:SetRoll(math.random(1, 100));
			end
		end
	else
		-- Rolling solo.
		local listItem = LyciusPlugins.RaidRolls.LPRollListItem(self.Player:GetName());
		listItem:SetSize(lbRolls:GetWidth(), 16);
		lbRolls:AddItem(listItem);
		--listItem:SetRoll(math.random(1, 100));
	end
end

function RaidRolls:FinalizeGroup()
	local maxItems = lbRolls:GetItemCount();
	local maxRoll = 0;
	local maxPos = 0;
	
	-- Selection sort.
	for i = 1, maxItems do
		maxPos = i;
		for j = i + 1, maxItems do
			if (lbRolls:GetItem(maxPos):GetRoll() < lbRolls:GetItem(j):GetRoll()) then
				maxPos = j;
			end
		end
		
		if (i ~= maxPos) then
			self:SwapItems(i, maxPos);
		end
		
		-- First iteration establishes max roll.
		if (maxRoll == 0) then
			maxRoll = lbRolls:GetItem(i):GetRoll();
		end
		
		if lbRolls:GetItem(i):HasRolled() then
			lbRolls:GetItem(i):SetReady();
			
			-- Highlight ones.
			if (lbRolls:GetItem(i):GetRoll() == 1) then
				lbRolls:GetItem(i):SetOne();
			end
			
			-- If this is the max roll, highlight it (including ties).
			if (lbRolls:GetItem(i):GetRoll() == maxRoll) then
				lbRolls:GetItem(i):SetWinner();
			end
		else
			lbRolls:GetItem(i):SetDisabled();
		end
	end
end

function RaidRolls:SwapItems(a, b)
	-- TODO: Find a (non-crashy) way to deal with references, rather than creating copies.
	local itemA = LyciusPlugins.RaidRolls.LPRollListItem(lbRolls:GetItem(a):GetName());
	itemA:SetSize(lbRolls:GetWidth(), 16);
	itemA:SetRoll(lbRolls:GetItem(a):GetRoll());

	local itemB = LyciusPlugins.RaidRolls.LPRollListItem(lbRolls:GetItem(b):GetName());
	itemB:SetSize(lbRolls:GetWidth(), 16);
	itemB:SetRoll(lbRolls:GetItem(b):GetRoll());
	
	lbRolls:RemoveItem(lbRolls:GetItem(a));
	lbRolls:InsertItem(a, itemB);
	lbRolls:RemoveItem(lbRolls:GetItem(b));
	lbRolls:InsertItem(b, itemA);
end

function RaidRolls:LoadSettings()
	-- Load settings.  Use default values if not set.
	self.settings = Turbine.PluginData.Load(Turbine.DataScope.Character, "RaidRolls");

	if (type(self.settings) ~= "table") then
		self.settings = { };
	end

	if not self.settings.posX then
		self.settings.posX = 50;
	end

	if not self.settings.posY then
		self.settings.posY = 50;
	end
end

function RaidRolls:SaveSettings()
	Turbine.PluginData.Save(Turbine.DataScope.Character, "RaidRolls", self.settings);
end
